@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2018, 2019.
@@ -14,8 +12,10 @@
 
 """IBM Quantum Experience API clients."""
 
-from .base import BaseClient
+from .base import BaseClient, WebsocketClientCloseCode
 from .account import AccountClient
 from .auth import AuthClient
 from .version import VersionClient
 from .websocket import WebsocketClient
+from .runtime import RuntimeClient
+from .runtime_ws import RuntimeWebsocketClient
