@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2018, 2020.
@@ -14,16 +12,14 @@
 
 """Client for accessing an individual IBM Quantum Experience account."""
 
-import asyncio
 import logging
 import time
+from threading import Timer
 
 from typing import List, Dict, Any, Optional, Union
-# Disabled unused-import because datetime is used only for type hints.
-from datetime import datetime  # pylint: disable=unused-import
+from datetime import datetime
 
-from qiskit.providers.ibmq.apiconstants import (API_JOB_FINAL_STATES, ApiJobStatus,
-                                                ApiJobShareLevel)
+from qiskit.providers.ibmq.apiconstants import API_JOB_FINAL_STATES, ApiJobStatus
 from qiskit.providers.ibmq.utils.utils import RefreshQueue
 from qiskit.providers.ibmq.credentials import Credentials
 
@@ -34,7 +30,7 @@ from ..rest.backend import Backend
 from ..session import RetrySession
 from ..exceptions import ApiIBMQProtocolError
 from .base import BaseClient
-from .websocket import WebsocketClient
+from .websocket import WebsocketClient, WebsocketClientCloseCode
 
 logger = logging.getLogger(__name__)
 
@@ -44,25 +40,23 @@ class AccountClient(BaseClient):
 
     def __init__(
             self,
-            access_token: str,
             credentials: Credentials,
             **request_kwargs: Any
     ) -> None:
         """AccountClient constructor.
 
         Args:
-            access_token: IBM Quantum Experience access token.
             credentials: Account credentials.
             **request_kwargs: Arguments for the request ``Session``.
         """
-        self._session = RetrySession(credentials.base_url, access_token, **request_kwargs)
+        self._session = RetrySession(
+            credentials.base_url, credentials.access_token, **request_kwargs)
         # base_api is used to handle endpoints that don't include h/g/p.
         # account_api is for h/g/p.
         self.base_api = Api(self._session)
         self.account_api = Account(session=self._session, hub=credentials.hub,
                                    group=credentials.group, project=credentials.project)
-        self.client_ws = WebsocketClient(credentials.websockets_url, access_token)
-        self._use_websockets = (not credentials.proxies)
+        self._credentials = credentials
 
     # Backend-related public functions.
 
@@ -185,8 +179,8 @@ class AccountClient(BaseClient):
             backend_name: str,
             qobj_dict: Dict[str, Any],
             job_name: Optional[str] = None,
-            job_share_level: Optional[ApiJobShareLevel] = None,
-            job_tags: Optional[List[str]] = None
+            job_tags: Optional[List[str]] = None,
+            experiment_id: Optional[str] = None
     ) -> Dict[str, Any]:
         """Submit a ``Qobj`` to the backend.
 
@@ -194,34 +188,41 @@ class AccountClient(BaseClient):
             backend_name: The name of the backend.
             qobj_dict: The ``Qobj`` to be executed, as a dictionary.
             job_name: Custom name to be assigned to the job.
-            job_share_level: Level the job should be shared at.
             job_tags: Tags to be assigned to the job.
+            experiment_id: Used to add a job to an experiment.
 
         Returns:
             Job data.
+
+        Raises:
+            RequestsApiError: If an error occurred communicating with the server.
         """
-        # Check for the job share level.
-        _job_share_level = job_share_level.value if job_share_level else None
+        # pylint: disable=missing-raises-doc
 
         # Create a remote job instance on the server.
         job_info = self.account_api.create_remote_job(
             backend_name,
             job_name=job_name,
-            job_share_level=_job_share_level,
-            job_tags=job_tags)
+            job_tags=job_tags,
+            experiment_id=experiment_id)
 
         # Get the upload URL.
         job_id = job_info['id']
         upload_url = job_info['objectStorageInfo']['uploadUrl']
         job_api = self.account_api.job(job_id)
 
-        # Upload the Qobj to object storage.
-        _ = job_api.put_object_storage(upload_url, qobj_dict)
-
-        # Notify the API via the callback.
-        response = job_api.callback_upload()
-
-        return response['job']
+        try:
+            # Upload the Qobj to object storage.
+            _ = job_api.put_object_storage(upload_url, qobj_dict)
+            # Notify the API via the callback.
+            response = job_api.callback_upload()
+            return response['job']
+        except Exception:  # pylint: disable=broad-except
+            try:
+                job_api.cancel()    # Cancel the job so it doesn't become a phantom job.
+            except Exception:  # pylint: disable=broad-except
+                pass
+            raise
 
     def job_download_qobj(self, job_id: str, use_object_storage: bool) -> Dict:
         """Retrieve and return a ``Qobj``.
@@ -355,21 +356,20 @@ class AccountClient(BaseClient):
         """
         status_response = None
         # Attempt to use websocket if available.
-        if self._use_websockets:
-            start_time = time.time()
-            try:
-                status_response = self._job_final_status_websocket(
-                    job_id=job_id, timeout=timeout, status_queue=status_queue)
-            except WebsocketTimeoutError as ex:
-                logger.info('Timeout checking job status using websocket, '
-                            'retrying using HTTP: %s', ex)
-            except (RuntimeError, WebsocketError) as ex:
-                logger.info('Error checking job status using websocket, '
-                            'retrying using HTTP: %s', ex)
+        start_time = time.time()
+        try:
+            status_response = self._job_final_status_websocket(
+                job_id=job_id, timeout=timeout, status_queue=status_queue)
+        except WebsocketTimeoutError as ex:
+            logger.info('Timeout checking job status using websocket, '
+                        'retrying using HTTP: %s', ex)
+        except (RuntimeError, WebsocketError) as ex:
+            logger.info('Error checking job status using websocket, '
+                        'retrying using HTTP: %s', ex)
 
-            # Adjust timeout for HTTP retry.
-            if timeout is not None:
-                timeout -= (time.time() - start_time)
+        # Adjust timeout for HTTP retry.
+        if timeout is not None:
+            timeout -= (time.time() - start_time)
 
         if not status_response:
             # Use traditional http requests if websocket not available or failed.
@@ -395,23 +395,26 @@ class AccountClient(BaseClient):
             Job status.
 
         Raises:
-            RuntimeError: If an unexpected error occurred while getting the event loop.
             WebsocketError: If the websocket connection ended unexpectedly.
             WebsocketTimeoutError: If the timeout has been reached.
         """
-        # As mentioned in `websocket.py`, in jupyter we need to use
-        # `nest_asyncio` to allow nested event loops.
+        ws_client = WebsocketClient(
+            websocket_url=self._credentials.websockets_url.rstrip('/'),
+            credentials=self._credentials,
+            job_id=job_id,
+            message_queue=status_queue
+        )
+        timer = None
+        if timeout:
+            timer = Timer(timeout, ws_client.disconnect,
+                          kwargs={'close_code': WebsocketClientCloseCode.TIMEOUT})
+            timer.start()
+
         try:
-            loop = asyncio.get_event_loop()
-        except RuntimeError as ex:
-            # Event loop may not be set in a child thread.
-            if 'There is no current event loop' in str(ex):
-                loop = asyncio.new_event_loop()
-                asyncio.set_event_loop(loop)
-            else:
-                raise
-        return loop.run_until_complete(
-            self.client_ws.get_job_status(job_id, timeout=timeout, status_queue=status_queue))
+            return ws_client.get_job_status()
+        finally:
+            if timer:
+                timer.cancel()
 
     def _job_final_status_polling(
             self,
@@ -497,3 +500,11 @@ class AccountClient(BaseClient):
             it is associated with.
         """
         return self.account_api.job(job_id).update_attribute({attr_name: attr_value})
+
+    def job_delete(self, job_id: str) -> None:
+        """Mark the job for deletion.
+
+        Args:
+            job_id: ID of the job to be deleted.
+        """
+        self.account_api.job(job_id).delete()
