@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2018, 2020.
@@ -16,7 +14,7 @@
 
 import json
 from typing import Dict, Optional, Any, List
-from datetime import datetime  # pylint: disable=unused-import
+from datetime import datetime
 
 from .base import RestAdapterBase
 from ..session import RetrySession
