@@ -162,6 +162,7 @@ class Api(RestAdapterBase):
             public_only: Optional[bool] = False,
             exclude_mine: Optional[bool] = False,
             mine_only: Optional[bool] = False,
+            parent_id: Optional[str] = None,
             sort_by: Optional[str] = None
     ) -> str:
         """Return experiment data.
@@ -181,6 +182,7 @@ class Api(RestAdapterBase):
             public_only: Whether or not to only return experiments with a public share level.
             exclude_mine: Whether or not to exclude experiments where I am the owner.
             mine_only: Whether or not to only return experiments where I am the owner.
+            parent_id: Filter by parent experiment ID.
             sort_by: Sorting order.
 
         Returns:
@@ -208,6 +210,8 @@ class Api(RestAdapterBase):
             params['group_id'] = group
         if project:
             params['project_id'] = project
+        if parent_id:
+            params['parent_experiment_uuid'] = parent_id
         if exclude_public:
             params['visibility'] = '!public'
         elif public_only:
