@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2017, 2019.
@@ -17,9 +15,9 @@
 import logging
 import os
 from ast import literal_eval
-from collections import OrderedDict
+from collections import defaultdict
 from configparser import ConfigParser, ParsingError
-from typing import Dict, Tuple, Optional, Any
+from typing import Dict, Tuple, Optional, Any, Union
 
 from .credentials import Credentials
 from .hubgroupproject import HubGroupProject
@@ -31,10 +29,15 @@ DEFAULT_QISKITRC_FILE = os.path.join(os.path.expanduser("~"),
                                      '.qiskit', 'qiskitrc')
 """Default location of the configuration file."""
 
+_ACTIVE_PREFERENCES = {
+    'experiment': {'auto_save': lambda val: val.lower() == 'true'}
+}
+_PREFERENCES_SECTION_NAME = 'ibmq.preferences'
+
 
 def read_credentials_from_qiskitrc(
         filename: Optional[str] = None
-) -> Tuple[Dict[HubGroupProject, Credentials], HubGroupProject]:
+) -> Tuple[Dict[HubGroupProject, Credentials], Dict]:
     """Read a configuration file and return a dictionary with its contents.
 
     Args:
@@ -42,10 +45,10 @@ def read_credentials_from_qiskitrc(
             location is used (``$HOME/.qiskit/qiskitrc``).
 
     Returns:
-        A tuple containing the found credentials, if any, and the default
-        provider stored, if specified in the configuration file. The format
+        A tuple containing the found credentials and the stored
+        preferences, if any, in the configuration file. The format
         for the found credentials is ``{credentials_unique_id: Credentials}``,
-        whereas the default provider is represented as a `HubGroupProject` instance.
+        whereas the preferences is ``{credentials_unique_id: {category: {key: val}}}``.
 
     Raises:
         InvalidCredentialsFormatError: If the file cannot be parsed. Note
@@ -56,6 +59,8 @@ def read_credentials_from_qiskitrc(
     """
     filename = filename or DEFAULT_QISKITRC_FILE
     config_parser = ConfigParser()
+    # Preserve case.
+    config_parser.optionxform = str  # type: ignore
     try:
         config_parser.read(filename)
     except ParsingError as ex:
@@ -63,37 +68,70 @@ def read_credentials_from_qiskitrc(
             'Error parsing file {}: {}'.format(filename, str(ex))) from ex
 
     # Build the credentials dictionary.
-    credentials_dict = OrderedDict()  # type: ignore[var-annotated]
-    default_provider_hgp = None
+    credentials_dict: Dict[HubGroupProject, Credentials] = {}
+    preferences: Dict[HubGroupProject, Dict] = {}
+
     for name in config_parser.sections():
-        single_credentials = dict(config_parser.items(name))
+        if not name.startswith('ibmq'):
+            continue
+
+        single_section = dict(config_parser.items(name))
+        configs: Dict[str, Union[str, bool, HubGroupProject, Dict]] = {}
+
+        if name == _PREFERENCES_SECTION_NAME:
+            preferences = _parse_preferences(single_section)
+            continue
 
         # Individually convert keys to their right types.
         # TODO: consider generalizing, moving to json configuration or a more
         # robust alternative.
-        if 'proxies' in single_credentials.keys():
-            single_credentials['proxies'] = literal_eval(
-                single_credentials['proxies'])
-        if 'verify' in single_credentials.keys():
-            single_credentials['verify'] = bool(  # type: ignore[assignment]
-                single_credentials['verify'])
-        if 'default_provider' in single_credentials.keys():
-            default_provider_hgp = HubGroupProject.from_stored_format(
-                single_credentials['default_provider'])
-
-            # Delete `default_provider`, since it's not used by the `Credentials` constructor.
-            del single_credentials['default_provider']
-
-        new_credentials = Credentials(**single_credentials)  # type: ignore[arg-type]
+        for key, val in single_section.items():
+            if key == 'proxies':
+                configs[key] = literal_eval(val)
+            elif key == 'verify':
+                configs[key] = config_parser[name].getboolean('verify')
+            elif key == 'default_provider':
+                configs[key] = HubGroupProject.from_stored_format(val)
+            else:
+                configs[key] = val
 
+        new_credentials = Credentials(**configs)  # type: ignore[arg-type]
         credentials_dict[new_credentials.unique_id()] = new_credentials
 
-    return credentials_dict, default_provider_hgp
+    return credentials_dict, preferences
+
+
+def _parse_preferences(pref_section: Dict) -> Dict[HubGroupProject, Dict]:
+    """Parse the preferences section.
+
+    Args:
+        pref_section: Preferences section.
+
+    Returns:
+        Parsed preferences, indexed by hub/group/project.
+    """
+    preferences: Dict[HubGroupProject, Dict] = defaultdict(dict)
+    for key, val in pref_section.items():
+        # Preferences section format is hgp,category,item=value
+        elems = key.split(',')
+        if len(elems) != 3:
+            continue
+        hgp, pref_cat, pref_key = elems
+        try:
+            val_type = _ACTIVE_PREFERENCES[pref_cat][pref_key]
+        except KeyError:
+            continue
+        hgp = HubGroupProject.from_stored_format(hgp)
+        cur_val = preferences[hgp].get(pref_cat, {})
+        cur_val.update({pref_key: val_type(val)})  # type: ignore[no-untyped-call]
+        preferences[hgp].update({pref_cat: cur_val})
+
+    return preferences
 
 
 def write_qiskit_rc(
         credentials: Dict[HubGroupProject, Credentials],
-        default_provider: Optional[HubGroupProject] = None,
+        preferences: Optional[Dict] = None,
         filename: Optional[str] = None
 ) -> None:
     """Write credentials to the configuration file.
@@ -101,14 +139,12 @@ def write_qiskit_rc(
     Args:
         credentials: Dictionary with the credentials, in the
             ``{credentials_unique_id: Credentials}`` format.
-        default_provider: If specified, the provider to store in the configuration
-            file, represented as a ``HubGroupProject`` instance.
+        preferences: Preferences to store.
         filename: Full path to the configuration file. If ``None``, the default
             location is used (``$HOME/.qiskit/qiskitrc``).
     """
     def _credentials_object_to_dict(
             credentials_obj: Credentials,
-            default_provider_to_store: Optional[HubGroupProject]
     ) -> Dict[str, Any]:
         """Convert a ``Credential`` object to a dictionary."""
         credentials_dict = {key: getattr(credentials_obj, key) for key in
@@ -116,8 +152,9 @@ def write_qiskit_rc(
                             if getattr(credentials_obj, key)}
 
         # Save the default provider to disk, if specified.
-        if default_provider_to_store:
-            credentials_dict['default_provider'] = default_provider_to_store.to_stored_format()
+        if credentials_obj.default_provider:
+            credentials_dict['default_provider'] = \
+                credentials_obj.default_provider.to_stored_format()
 
         return credentials_dict
 
@@ -135,20 +172,31 @@ def write_qiskit_rc(
 
     unrolled_credentials = {
         _section_name(credentials_object):
-            _credentials_object_to_dict(credentials_object, default_provider)
+            _credentials_object_to_dict(credentials_object)
         for _, credentials_object in credentials.items()
     }
 
+    if preferences:
+        unrolled_pref = {}
+        # Preferences section format is hgp,category,key=value.
+        # Input preferences format is {hgp: {cat: {pref_key: pref_val}}}.
+        for hgp, hgp_val in preferences.items():
+            hgp = hgp.to_stored_format()
+            for cat, cat_val in hgp_val.items():
+                for pref, pref_val in cat_val.items():
+                    unrolled_pref[f"{hgp},{cat},{pref}"] = pref_val
+        unrolled_credentials[_PREFERENCES_SECTION_NAME] = unrolled_pref
+
     # Write the configuration file.
     with open(filename, 'w') as config_file:
         config_parser = ConfigParser()
+        config_parser.optionxform = str  # type: ignore
         config_parser.read_dict(unrolled_credentials)
         config_parser.write(config_file)
 
 
 def store_credentials(
         credentials: Credentials,
-        default_provider: Optional[HubGroupProject] = None,
         overwrite: bool = False,
         filename: Optional[str] = None
 ) -> None:
@@ -156,15 +204,13 @@ def store_credentials(
 
     Args:
         credentials: Credentials to save.
-        default_provider: If specified, the provider to store in the configuration
-            file, represented as a ``HubGroupProject`` instance.
         overwrite: ``True`` if any existing credentials are to be overwritten.
         filename: Full path to the configuration file. If ``None``, the default
             location is used (``$HOME/.qiskit/qiskitrc``).
     """
     # Read the current providers stored in the configuration file.
     filename = filename or DEFAULT_QISKITRC_FILE
-    stored_credentials, _ = read_credentials_from_qiskitrc(filename)
+    stored_credentials, stored_preferences = read_credentials_from_qiskitrc(filename)
 
     # Check if duplicated credentials are already stored. By convention,
     # we assume (hub, group, project) is always unique.
@@ -175,7 +221,8 @@ def store_credentials(
 
     # Append and write the credentials to file.
     stored_credentials[credentials.unique_id()] = credentials
-    write_qiskit_rc(stored_credentials, default_provider, filename)
+    write_qiskit_rc(credentials=stored_credentials, preferences=stored_preferences,
+                    filename=filename)
 
 
 def remove_credentials(
@@ -194,11 +241,37 @@ def remove_credentials(
             configuration file.
     """
     # Set the name of the Provider from the class.
-    stored_credentials, _ = read_credentials_from_qiskitrc(filename)
+    stored_credentials, stored_preferences = read_credentials_from_qiskitrc(filename)
 
     try:
         del stored_credentials[credentials.unique_id()]
     except KeyError:
         raise CredentialsNotFoundError('The account {} does not exist in the configuration file.'
                                        .format(credentials.unique_id())) from None
-    write_qiskit_rc(stored_credentials, filename)
+    write_qiskit_rc(credentials=stored_credentials, preferences=stored_preferences,
+                    filename=filename)
+
+
+def store_preferences(
+        preferences: Dict[HubGroupProject, Dict],
+        filename: Optional[str] = None
+) -> None:
+    """Store the preferences in the configuration file.
+
+    Args:
+        preferences: Preferences to save.
+        filename: Full path to the configuration file. If ``None``, the default
+            location is used (``$HOME/.qiskit/qiskitrc``).
+    """
+    # Read the current providers stored in the configuration file.
+    filename = filename or DEFAULT_QISKITRC_FILE
+    stored_credentials, stored_preferences = read_credentials_from_qiskitrc(filename)
+
+    # Merge with existing preferences at category level.
+    for hgp, hgp_val in preferences.items():
+        merged_hgp = stored_preferences.get(hgp, {})
+        merged_hgp.update(hgp_val)
+        stored_preferences[hgp] = merged_hgp
+
+    write_qiskit_rc(credentials=stored_credentials, preferences=stored_preferences,
+                    filename=filename)
