@@ -34,7 +34,7 @@ from qiskit.providers.models import (BackendStatus, BackendProperties,
 from qiskit.tools.events.pubsub import Publisher
 from qiskit.providers.models import (QasmBackendConfiguration,
                                      PulseBackendConfiguration)
-from qiskit.util import deprecate_arguments
+from qiskit.utils import deprecate_arguments
 
 from qiskit.providers.ibmq import accountprovider  # pylint: disable=unused-import
 from .apiconstants import ApiJobStatus, API_JOB_FINAL_STATES
@@ -135,7 +135,7 @@ class IBMQBackend(Backend):
     @classmethod
     def _default_options(cls) -> Options:
         """Default runtime options."""
-        return Options(shots=1024, memory=False,
+        return Options(shots=4000, memory=False,
                        qubit_lo_freq=None, meas_lo_freq=None,
                        schedule_los=None,
                        meas_level=MeasLevel.CLASSIFIED,
@@ -207,7 +207,7 @@ class IBMQBackend(Backend):
             header: User input that will be attached to the job and will be
                 copied to the corresponding result header. Headers do not affect the run.
                 This replaces the old ``Qobj`` header.
-            shots: Number of repetitions of each circuit, for sampling. Default: 1024
+            shots: Number of repetitions of each circuit, for sampling. Default: 4000
                 or ``max_shots`` from the backend configuration, whichever is smaller.
             memory: If ``True``, per-shot measurement bitstrings are returned as well
                 (provided the backend supports it). For OpenPulse jobs, only
