@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2019, 2020.
@@ -17,11 +15,13 @@
 import logging
 import warnings
 import copy
+from functools import wraps
 
 from typing import Dict, List, Callable, Optional, Any, Union
 from datetime import datetime
 
-from qiskit.providers import JobStatus, QiskitBackendNotFoundError  # type: ignore[attr-defined]
+from qiskit.providers.jobstatus import JobStatus
+from qiskit.providers.exceptions import QiskitBackendNotFoundError
 from qiskit.providers.providerutils import filter_backends
 from qiskit.providers.ibmq import accountprovider  # pylint: disable=unused-import
 
@@ -48,17 +48,17 @@ class IBMQBackendService:
     a specific backend::
 
         backends = provider.backends()  # Invoke backends() to get the backends.
-        sim_backend = provider.backends.ibmq_qasm_simulator  # Get a specific backend instance.
+        sim_backend = provider.backend.ibmq_qasm_simulator  # Get a specific backend instance.
 
     Also, you are able to retrieve jobs from a provider without specifying the backend name.
     For example, to retrieve the ten most recent jobs you have submitted, regardless of the
     backend they were submitted to, you could do::
 
-        most_recent_jobs = provider.backends.jobs(limit=10)
+        most_recent_jobs = provider.backend.jobs(limit=10)
 
     It is also possible to retrieve a single job without specifying the backend name::
 
-        job = provider.backends.retrieve_job(<JOB_ID>)
+        job = provider.backend.retrieve_job(<JOB_ID>)
     """
 
     def __init__(self, provider: 'accountprovider.AccountProvider') -> None:
@@ -83,11 +83,13 @@ class IBMQBackendService:
 
             setattr(self, backend_name, backend)
 
-    def __call__(
+    def backends(
             self,
             name: Optional[str] = None,
             filters: Optional[Callable[[List[IBMQBackend]], bool]] = None,
             timeout: Optional[float] = None,
+            min_num_qubits: Optional[int] = None,
+            input_allowed: Optional[Union[str, List[str]]] = None,
             **kwargs: Any
     ) -> List[IBMQBackend]:
         """Return all backends accessible via this provider, subject to optional filtering.
@@ -97,9 +99,16 @@ class IBMQBackendService:
             filters: More complex filters, such as lambda functions.
                 For example::
 
-                    AccountProvider.backends(filters=lambda b: b.configuration().n_qubits > 5)
+                    AccountProvider.backends(
+                        filters=lambda b: b.configuration().quantum_volume > 16)
             timeout: Maximum number of seconds to wait for the discovery of
                 remote backends.
+            min_num_qubits: Minimum number of qubits the backend has to have.
+            input_allowed: Filter by the types of input the backend supports.
+                Valid input types are ``job`` (circuit job) and ``runtime`` (Qiskit Runtime).
+                For example, ``inputs_allowed='runtime'`` will return all backends
+                that support Qiskit Runtime. If a list is given, the backend must
+                support all types specified in the list.
             kwargs: Simple filters that specify a ``True``/``False`` criteria in the
                 backend configuration, backends status, or provider credentials.
                 An example to get the operational backends with 5 qubits::
@@ -109,7 +118,12 @@ class IBMQBackendService:
         Returns:
             The list of available backends that match the filter.
         """
-        backends = self._provider._backends.values()
+        if timeout:
+            warnings.warn("The `timeout` keyword argument is deprecated and will "
+                          "be removed in a future release.",
+                          DeprecationWarning, stacklevel=2)
+
+        backends = list(self._provider._backends.values())
 
         # Special handling of the `name` parameter, to support alias
         # resolution.
@@ -119,6 +133,16 @@ class IBMQBackendService:
             name = aliases.get(name, name)
             kwargs['backend_name'] = name
 
+        if min_num_qubits:
+            backends = list(filter(
+                lambda b: b.configuration().n_qubits >= min_num_qubits, backends))
+
+        if input_allowed:
+            if not isinstance(input_allowed, list):
+                input_allowed = [input_allowed]
+            backends = list(filter(
+                lambda b: set(input_allowed) <= set(b.configuration().input_allowed), backends))
+
         return filter_backends(backends, filters=filters, **kwargs)
 
     def jobs(
@@ -132,6 +156,7 @@ class IBMQBackendService:
             end_datetime: Optional[datetime] = None,
             job_tags: Optional[List[str]] = None,
             job_tags_operator: Optional[str] = "OR",
+            experiment_id: Optional[str] = None,
             descending: bool = True,
             db_filter: Optional[Dict[str, Any]] = None
     ) -> List[IBMQJob]:
@@ -167,6 +192,7 @@ class IBMQBackendService:
                       specified in ``job_tags`` to be included.
                     * If "OR" is specified, then a job only needs to have any
                       of the tags specified in ``job_tags`` to be included.
+            experiment_id: Filter by job experiment ID.
             descending: If ``True``, return the jobs in descending order of the job
                 creation date (i.e. newest first) until the limit is reached.
             db_filter: A `loopback-based filter
@@ -204,12 +230,7 @@ class IBMQBackendService:
         if job_name:
             api_filter['name'] = {"regexp": job_name}
 
-        # TODO: Remove when decided the warning is no longer needed.
         if start_datetime or end_datetime:
-            warnings.warn('Unless a UTC timezone information is present, the parameters '
-                          '`start_datetime` and `end_datetime` are now expected to be in '
-                          'local time instead of UTC.', stacklevel=2)
-
             api_filter['creationDate'] = self._update_creation_date_filter(
                 cur_dt_filter={},
                 gte_dt=local_to_utc(start_datetime).isoformat() if start_datetime else None,
@@ -230,6 +251,9 @@ class IBMQBackendService:
                     '"{}" is not a valid job_tags_operator value. '
                     'Valid values are "AND" and "OR"'.format(job_tags_operator))
 
+        if experiment_id:
+            api_filter['experimentTag'] = experiment_id
+
         if db_filter:
             # Rather than overriding the logical operators `and`/`or`, first
             # check to see if the `api_filter` query should be extended with the
@@ -359,13 +383,13 @@ class IBMQBackendService:
                 lt_list.append(cur_dt_filter.pop('between')[1])
             lte_dt = min(lt_list) if lt_list else None
 
-        new_dt_filter = {}
+        new_dt_filter = {}  # type: Dict[str, Union[str, List[str]]]
         if gte_dt and lte_dt:
             new_dt_filter['between'] = [gte_dt, lte_dt]
         elif gte_dt:
-            new_dt_filter['gte'] = gte_dt  # type: ignore[assignment]
+            new_dt_filter['gte'] = gte_dt
         elif lte_dt:
-            new_dt_filter['lte'] = lte_dt  # type: ignore[assignment]
+            new_dt_filter['lte'] = lte_dt
 
         return new_dt_filter
 
@@ -431,7 +455,7 @@ class IBMQBackendService:
         elif status == JobStatus.DONE:
             _status_filter = {'status': ApiJobStatus.COMPLETED.value}
         elif status == JobStatus.ERROR:
-            _status_filter = {'status': {'regexp': '^ERROR'}}  # type: ignore[assignment]
+            _status_filter = {'status': {'regexp': '^ERROR'}}
         else:
             raise IBMQBackendValueError(
                 '"{}" is not a valid status value. Valid values are {}'.format(
@@ -505,3 +529,53 @@ class IBMQBackendService:
             'ibmq_16_rueschlikon': 'ibmqx5',
             'ibmq_20_austin': 'QS1_1'
             }
+
+
+def _issue_warning(func):  # type: ignore
+    @wraps(func)
+    def _wrapper(self, *args, **kwargs):  # type: ignore
+        if not self._backends_warning_issued:
+            warnings.warn("The `backends` attribute is deprecated. "
+                          "Please use `provider.backend` (singular) instead.",
+                          DeprecationWarning, stacklevel=2)
+            self._backends_warning_issued = True
+        return func(self, *args, **kwargs)
+    return _wrapper
+
+
+class IBMQDeprecatedBackendService:
+
+    # pylint: disable=W,C,R
+
+    def __init__(self, backend_service: IBMQBackendService):
+        self._backend_service = backend_service
+        self._backends_warning_issued = False
+
+    @_issue_warning
+    def jobs(self, *args, **kwargs):  # type: ignore
+        return self._backend_service.jobs(*args, **kwargs)
+
+    @_issue_warning
+    def retrieve_job(self, *args, **kwargs):  # type: ignore
+        return self._backend_service.retrieve_job(*args, **kwargs)
+
+    @_issue_warning
+    def my_reservations(self, *args, **kwargs):  # type: ignore
+        return self._backend_service.my_reservations(*args, **kwargs)
+
+    def __getattribute__(self, item):  # type: ignore
+        if item in ['_backend_service', '_backends_warning_issued']:
+            return super().__getattribute__(item)
+
+        if not self._backends_warning_issued:
+            warnings.warn("The `backends` provider attribute is deprecated. "
+                          "Please use `provider.backend` (singular) instead. "
+                          "You can continue to use `provider.backends()` to "
+                          "retrieve all backends.",
+                          DeprecationWarning, stacklevel=2)
+            self._backends_warning_issued = True
+
+        return self._backend_service.__getattribute__(item)
+
+    def __call__(self, *args, **kwargs):  # type: ignore
+        return self._backend_service.backends(*args, **kwargs)
