@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2017, 2019.
@@ -15,9 +13,9 @@
 """Factory and Account manager for IBM Quantum Experience."""
 
 import logging
-import warnings
 from typing import Dict, List, Union, Callable, Optional, Any
 from collections import OrderedDict
+import traceback
 
 from .accountprovider import AccountProvider
 from .api.clients import AuthClient, VersionClient
@@ -105,10 +103,6 @@ class IBMQFactory:
                 'The URL specified ({}) is not an IBM Quantum Experience authentication '
                 'URL. Valid authentication URL: {}.'.format(credentials.url, QX_AUTH_URL))
 
-        # TODO Remove in the future.
-        warnings.warn('Timestamps in IBMQ backend properties, jobs, and job results '
-                      'are all now in local time instead of UTC.')
-
         # Initialize the providers.
         self._initialize_providers(credentials)
 
@@ -162,11 +156,10 @@ class IBMQFactory:
         """
         # Check for valid credentials.
         try:
-            stored_credentials, stored_provider_hgp = discover_credentials()
+            stored_credentials, preferences = discover_credentials()
         except HubGroupProjectInvalidStateError as ex:
             raise IBMQAccountCredentialsInvalidFormat(
-                'The default provider (hub/group/project) stored on disk could not '
-                'be parsed: {}'.format(str(ex))) from ex
+                'Invalid provider (hub/group/project) data found {}'.format(str(ex))) from ex
 
         credentials_list = list(stored_credentials.values())
 
@@ -188,10 +181,6 @@ class IBMQFactory:
             raise IBMQAccountCredentialsInvalidUrl(
                 'Invalid IBM Quantum Experience credentials found. ' + UPDATE_ACCOUNT_TEXT)
 
-        # TODO: Remove in the future.
-        warnings.warn('Timestamps in IBMQ backend properties, jobs, and job results '
-                      'are all now in local time instead of UTC.')
-
         # Initialize the providers.
         if self._credentials:
             # For convention, emit a warning instead of raising.
@@ -199,7 +188,7 @@ class IBMQFactory:
                            'account in the session will be replaced.')
             self.disable_account()
 
-        self._initialize_providers(credentials)
+        self._initialize_providers(credentials, preferences)
 
         # Prevent edge case where no hubs are available.
         providers = self.providers()
@@ -211,8 +200,8 @@ class IBMQFactory:
         default_provider = providers[0]
 
         # If specified, attempt to get the provider stored for the account.
-        if stored_provider_hgp:
-            hub, group, project = stored_provider_hgp.to_tuple()
+        if credentials.default_provider:
+            hub, group, project = credentials.default_provider.to_tuple()
             try:
                 default_provider = self.get_provider(hub=hub, group=group, project=project)
             except IBMQProviderError as ex:
@@ -269,8 +258,6 @@ class IBMQFactory:
                 'Invalid IBM Quantum Experience token '
                 'found: "{}" of type {}.'.format(token, type(token)))
 
-        credentials = Credentials(token, url, **kwargs)
-
         # If any `hub`, `group`, or `project` is specified, make sure all parameters are set.
         if any([hub, group, project]) and not all([hub, group, project]):
             raise IBMQAccountValueError('The hub, group, and project parameters must all be '
@@ -282,8 +269,10 @@ class IBMQFactory:
         default_provider_hgp = HubGroupProject(hub, group, project) \
             if all([hub, group, project]) else None
 
+        credentials = Credentials(token=token, url=url,
+                                  default_provider=default_provider_hgp, **kwargs)
+
         store_credentials(credentials,
-                          default_provider=default_provider_hgp,
                           overwrite=overwrite)
 
     @staticmethod
@@ -454,36 +443,44 @@ class IBMQFactory:
                                        **credentials.connection_parameters())
         return version_finder.version()
 
-    def _initialize_providers(self, credentials: Credentials) -> None:
-        """Authenticate against IBM Quantum Experience and populate the providers.
+    def _initialize_providers(
+            self, credentials: Credentials,
+            preferences: Optional[Dict] = None
+    ) -> None:
+        """Authenticate against IBM Quantum and populate the providers.
 
         Args:
-            credentials: Credentials for IBM Quantum Experience.
+            credentials: Credentials for IBM Quantum.
+            preferences: Account preferences.
         """
         auth_client = AuthClient(credentials.token,
                                  credentials.base_url,
                                  **credentials.connection_parameters())
         service_urls = auth_client.current_service_urls()
         user_hubs = auth_client.user_hubs()
+        preferences = preferences or {}
 
         self._credentials = credentials
         for hub_info in user_hubs:
             # Build credentials.
             provider_credentials = Credentials(
                 credentials.token,
+                access_token=auth_client.current_access_token(),
                 url=service_urls['http'],
                 websockets_url=service_urls['ws'],
                 proxies=credentials.proxies,
                 verify=credentials.verify,
                 services=service_urls.get('services', {}),
+                default_provider=credentials.default_provider,
                 **hub_info, )
+            provider_credentials.preferences = \
+                preferences.get(provider_credentials.unique_id(), {})
 
             # Build the provider.
             try:
-                provider = AccountProvider(provider_credentials,
-                                           auth_client.current_access_token())
+                provider = AccountProvider(provider_credentials, self)
                 self._providers[provider_credentials.unique_id()] = provider
-            except Exception as ex:  # pylint: disable=broad-except
+            except Exception:  # pylint: disable=broad-except
                 # Catch-all for errors instantiating the provider.
                 logger.warning('Unable to instantiate provider for %s: %s',
-                               hub_info, ex)
+                               hub_info, traceback.format_exc())
