@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2020.
@@ -15,7 +13,7 @@
 """IBMQ random number service."""
 
 import logging
-from typing import Dict, List
+from typing import Dict, List, Any
 
 from qiskit.providers.ibmq import accountprovider  # pylint: disable=unused-import
 from .baserandomservice import BaseRandomService
@@ -41,16 +39,15 @@ class IBMQRandomService:
         extractor = provider.random.cqc_extractor  # Short hand for above.
     """
 
-    def __init__(self, provider: 'accountprovider.AccountProvider', access_token: str) -> None:
+    def __init__(self, provider: 'accountprovider.AccountProvider') -> None:
         """IBMQRandomService constructor.
 
         Args:
             provider: IBM Quantum Experience account provider.
-            access_token: IBM Quantum Experience access token.
         """
         self._provider = provider
         if provider.credentials.extractor_url:
-            self._random_client = RandomClient(access_token, provider.credentials)
+            self._random_client = RandomClient(provider.credentials)
             self._initialized = False
         else:
             self._random_client = None
@@ -105,12 +102,13 @@ class IBMQRandomService:
 
         return service
 
-    def __dir__(self):
+    def __dir__(self) -> Dict:
         self._discover_services()
         return self.__dict__
 
-    def __getattr__(self, item):
-        self._discover_services()
+    def __getattr__(self, item: str) -> Any:
+        if not self.__getattribute__('_initialized'):
+            self._discover_services()
         try:
             return self._services[item]
         except KeyError:
