@@ -15,14 +15,15 @@
 import logging
 from typing import Dict, Callable, Optional, Union, List, Any, Type
 import json
-import copy
+import re
+import warnings
 
 from qiskit.providers.exceptions import QiskitBackendNotFoundError
 from qiskit.providers.ibmq import accountprovider  # pylint: disable=unused-import
 
 from .runtime_job import RuntimeJob
-from .runtime_program import RuntimeProgram, ProgramParameter, ProgramResult, ParameterNamespace
-from .utils import RuntimeEncoder, RuntimeDecoder
+from .runtime_program import RuntimeProgram, ParameterNamespace
+from .utils import RuntimeDecoder, to_base64_string
 from .exceptions import (QiskitRuntimeError, RuntimeDuplicateProgramError, RuntimeProgramNotFound,
                          RuntimeJobNotFound)
 from .program.result_decoder import ResultDecoder
@@ -106,19 +107,30 @@ class IBMRuntimeService:
         self._ws_url = provider.credentials.runtime_url.replace('https', 'wss')
         self._programs = {}  # type: Dict
 
-    def pprint_programs(self, refresh: bool = False) -> None:
+    def pprint_programs(self, refresh: bool = False, detailed: bool = False,
+                        limit: int = 20, skip: int = 0) -> None:
         """Pretty print information about available runtime programs.
 
         Args:
             refresh: If ``True``, re-query the server for the programs. Otherwise
                 return the cached value.
+            detailed: If ``True`` print all details about available runtime programs.
+            limit: The number of programs returned at a time. Default and maximum
+                value of 20.
+            skip: The number of programs to skip.
         """
-        programs = self.programs(refresh)
+        programs = self.programs(refresh, limit, skip)
         for prog in programs:
             print("="*50)
-            print(str(prog))
+            if detailed:
+                print(str(prog))
+            else:
+                print(f"{prog.program_id}:",)
+                print(f"  Name: {prog.name}")
+                print(f"  Description: {prog.description}")
 
-    def programs(self, refresh: bool = False) -> List[RuntimeProgram]:
+    def programs(self, refresh: bool = False,
+                 limit: int = 20, skip: int = 0) -> List[RuntimeProgram]:
         """Return available runtime programs.
 
         Currently only program metadata is returned.
@@ -126,17 +138,34 @@ class IBMRuntimeService:
         Args:
             refresh: If ``True``, re-query the server for the programs. Otherwise
                 return the cached value.
+            limit: The number of programs returned at a time. ``None`` means no limit.
+            skip: The number of programs to skip.
 
         Returns:
             A list of runtime programs.
         """
+        if skip is None:
+            skip = 0
         if not self._programs or refresh:
             self._programs = {}
-            response = self._api_client.list_programs()
-            for prog_dict in response:
-                program = self._to_program(prog_dict)
-                self._programs[program.program_id] = program
-        return list(self._programs.values())
+            current_page_limit = 20
+            offset = 0
+            while True:
+                response = self._api_client.list_programs(limit=current_page_limit, skip=offset)
+                program_page = response.get("programs", [])
+                # count is the total number of programs that would be returned if
+                # there was no limit or skip
+                count = response.get("count", 0)
+                for prog_dict in program_page:
+                    program = self._to_program(prog_dict)
+                    self._programs[program.program_id] = program
+                if len(self._programs) == count:
+                    # Stop if there are no more programs returned by the server.
+                    break
+                offset += len(program_page)
+        if limit is None:
+            limit = len(self._programs)
+        return list(self._programs.values())[skip:limit+skip]
 
     def program(self, program_id: str, refresh: bool = False) -> RuntimeProgram:
         """Retrieve a runtime program.
@@ -176,22 +205,29 @@ class IBMRuntimeService:
         Returns:
             A ``RuntimeProgram`` instance.
         """
-        backend_req = json.loads(response.get('backendRequirements', '{}'))
-        params = json.loads(response.get('parameters', '{}')).get("doc", [])
-        ret_vals = json.loads(response.get('returnValues', '{}'))
-        interim_results = json.loads(response.get('interimResults', '{}'))
+        backend_requirements = {}
+        parameters = {}
+        return_values = {}
+        interim_results = {}
+        if "spec" in response:
+            backend_requirements = response["spec"].get('backend_requirements', {})
+            parameters = response["spec"].get('parameters', {})
+            return_values = response["spec"].get('return_values', {})
+            interim_results = response["spec"].get('interim_results', {})
 
         return RuntimeProgram(program_name=response['name'],
                               program_id=response['id'],
                               description=response.get('description', ""),
-                              parameters=params,
-                              return_values=ret_vals,
+                              parameters=parameters,
+                              return_values=return_values,
                               interim_results=interim_results,
                               max_execution_time=response.get('cost', 0),
-                              creation_date=response.get('creationDate', ""),
-                              version=response.get('version', "0"),
-                              backend_requirements=backend_req,
-                              is_public=response.get('isPublic', False))
+                              creation_date=response.get('creation_date', ""),
+                              update_date=response.get('update_date', ""),
+                              backend_requirements=backend_requirements,
+                              is_public=response.get('is_public', False),
+                              data=response.get('data', ""),
+                              api_client=self._api_client)
 
     def run(
             self,
@@ -199,7 +235,8 @@ class IBMRuntimeService:
             options: Dict,
             inputs: Union[Dict, ParameterNamespace],
             callback: Optional[Callable] = None,
-            result_decoder: Optional[Type[ResultDecoder]] = None
+            result_decoder: Optional[Type[ResultDecoder]] = None,
+            image: Optional[str] = ""
     ) -> RuntimeJob:
         """Execute the runtime program.
 
@@ -217,6 +254,8 @@ class IBMRuntimeService:
 
             result_decoder: A :class:`ResultDecoder` subclass used to decode job results.
                 ``ResultDecoder`` is used if not specified.
+            image: The runtime image used to execute the program, specified in the form
+                of image_name:tag. Not all accounts are authorized to select a different image.
 
         Returns:
             A ``RuntimeJob`` instance representing the execution.
@@ -231,13 +270,18 @@ class IBMRuntimeService:
             inputs.validate()
             inputs = vars(inputs)
 
+        if image and not \
+            re.match("[a-zA-Z0-9]+([/.\\-_][a-zA-Z0-9]+)*:[a-zA-Z0-9]+([.\\-_][a-zA-Z0-9]+)*$",
+                     image):
+            raise IBMQInputValueError('"image" needs to be in form of image_name:tag')
+
         backend_name = options['backend_name']
-        params_str = json.dumps(inputs, cls=RuntimeEncoder)
         result_decoder = result_decoder or ResultDecoder
         response = self._api_client.program_run(program_id=program_id,
                                                 credentials=self._provider.credentials,
                                                 backend_name=backend_name,
-                                                params=params_str)
+                                                params=inputs,
+                                                image=image)
 
         backend = self._provider.get_backend(backend_name)
         job = RuntimeJob(backend=backend,
@@ -245,21 +289,14 @@ class IBMRuntimeService:
                          credentials=self._provider.credentials,
                          job_id=response['id'], program_id=program_id, params=inputs,
                          user_callback=callback,
-                         result_decoder=result_decoder)
+                         result_decoder=result_decoder,
+                         image=image)
         return job
 
     def upload_program(
             self,
-            data: Union[bytes, str],
-            metadata: Optional[Union[Dict, str]] = None,
-            name: Optional[str] = None,
-            max_execution_time: Optional[int] = None,
-            description: Optional[str] = None,
-            version: Optional[float] = None,
-            backend_requirements: Optional[str] = None,
-            parameters: Optional[List[ProgramParameter]] = None,
-            return_values: Optional[List[ProgramResult]] = None,
-            interim_results: Optional[List[ProgramResult]] = None
+            data: str,
+            metadata: Optional[Union[Dict, str]] = None
     ) -> str:
         """Upload a runtime program.
 
@@ -280,19 +317,25 @@ class IBMRuntimeService:
         ``name2`` will be used as the program name.
 
         Args:
-            data: Name of the program file or program data to upload.
+            data: Program data or path of the file containing program data to upload.
             metadata: Name of the program metadata file or metadata dictionary.
-                A metadata file needs to be in the JSON format.
-                See :file:`program/program_metadata_sample.yaml` for an example.
-            name: Name of the program. Required if not specified via `metadata`.
-            max_execution_time: Maximum execution time in seconds. Required if
-                not specified via `metadata`.
-            description: Program description. Required if not specified via `metadata`.
-            version: Program version. The default is 1.0 if not specified.
-            backend_requirements: Backend requirements.
-            parameters: A list of program input parameters.
-            return_values: A list of program return values.
-            interim_results: A list of program interim results.
+                A metadata file needs to be in the JSON format. The ``parameters``,
+                ``return_values``, and ``interim_results`` should be defined as JSON Schema.
+                See :file:`program/program_metadata_sample.json` for an example. The
+                fields in metadata are explained below.
+
+                * name: Name of the program. Required.
+                * max_execution_time: Maximum execution time in seconds. Required.
+                * description: Program description. Required.
+                * is_public: Whether the runtime program should be visible to the public.
+                                    The default is ``False``.
+                * spec: Specifications for backend characteristics and input parameters
+                    required to run the program, interim results and final result.
+
+                    * backend_requirements: Backend requirements.
+                    * parameters: Program input parameters in JSON schema format.
+                    * return_values: Program return values in JSON schema format.
+                    * interim_results: Program interim results in JSON schema format.
 
         Returns:
             Program ID.
@@ -303,20 +346,21 @@ class IBMRuntimeService:
             IBMQNotAuthorizedError: If you are not authorized to upload programs.
             QiskitRuntimeError: If the upload failed.
         """
-        program_metadata = self._merge_metadata(
-            initial={},
-            metadata=metadata,
-            name=name, max_execution_time=max_execution_time, description=description,
-            version=version, backend_requirements=backend_requirements,
-            parameters=parameters,
-            return_values=return_values, interim_results=interim_results)
+        program_metadata = self._read_metadata(metadata=metadata)
 
         for req in ['name', 'description', 'max_execution_time']:
             if req not in program_metadata or not program_metadata[req]:
                 raise IBMQInputValueError(f"{req} is a required metadata field.")
 
+        if "def main(" not in data:
+            # This is the program file
+            with open(data, "r") as file:
+                data = file.read()
+
         try:
-            response = self._api_client.program_create(program_data=data, **program_metadata)
+            program_data = to_base64_string(data)
+            response = self._api_client.program_create(program_data=program_data,
+                                                       **program_metadata)
         except RequestsApiError as ex:
             if ex.status_code == 409:
                 raise RuntimeDuplicateProgramError(
@@ -327,21 +371,17 @@ class IBMRuntimeService:
             raise QiskitRuntimeError(f"Failed to create program: {ex}") from None
         return response['id']
 
-    def _merge_metadata(
+    def _read_metadata(
             self,
-            initial: Dict,
-            metadata: Optional[Union[Dict, str]] = None,
-            **kwargs: Any
+            metadata: Optional[Union[Dict, str]] = None
     ) -> Dict:
-        """Merge multiple copies of metadata.
+        """Read metadata.
 
         Args:
-            initial: The initial metadata. This may be mutated.
             metadata: Name of the program metadata file or metadata dictionary.
-            **kwargs: Additional metadata fields to overwrite.
 
         Returns:
-            Merged metadata.
+            Return metadata.
         """
         upd_metadata: dict = {}
         if metadata is not None:
@@ -349,33 +389,95 @@ class IBMRuntimeService:
                 with open(metadata, 'r') as file:
                     upd_metadata = json.load(file)
             else:
-                upd_metadata = copy.deepcopy(metadata)
-
-        self._tuple_to_dict(initial)
-        initial.update(upd_metadata)
-
-        self._tuple_to_dict(kwargs)
-        for key, val in kwargs.items():
-            if val is not None:
-                initial[key] = val
-
+                upd_metadata = metadata
         # TODO validate metadata format
-        metadata_keys = ['name', 'max_execution_time', 'description', 'version',
-                         'backend_requirements', 'parameters', 'return_values',
-                         'interim_results']
-        return {key: val for key, val in initial.items() if key in metadata_keys}
+        metadata_keys = ['name', 'max_execution_time', 'description',
+                         'spec', 'is_public']
+        return {key: val for key, val in upd_metadata.items() if key in metadata_keys}
 
-    def _tuple_to_dict(self, metadata: Dict) -> None:
-        """Convert fields in metadata from named tuples to dictionaries.
+    def update_program(
+            self,
+            program_id: str,
+            data: str = None,
+            metadata: Optional[Union[Dict, str]] = None,
+            name: str = None,
+            description: str = None,
+            max_execution_time: int = None,
+            spec: Optional[Dict] = None
+    ) -> None:
+        """Update a runtime program.
+
+        Program metadata can be specified using the `metadata` parameter or
+        individual parameters, such as `name` and `description`. If the
+        same metadata field is specified in both places, the individual parameter
+        takes precedence.
 
         Args:
-            metadata: Metadata to be converted.
+            program_id: Program ID.
+            data: Program data or path of the file containing program data to upload.
+            metadata: Name of the program metadata file or metadata dictionary.
+            name: New program name.
+            description: New program description.
+            max_execution_time: New maximum execution time.
+            spec: New specifications for backend characteristics, input parameters,
+                interim results and final result.
+
+        Raises:
+            RuntimeProgramNotFound: If the program doesn't exist.
+            QiskitRuntimeError: If the request failed.
         """
-        for key in ['parameters', 'return_values', 'interim_results']:
-            doc_list = metadata.pop(key, None)
-            if not doc_list or isinstance(doc_list[0], dict):
-                continue
-            metadata[key] = [dict(elem._asdict()) for elem in doc_list]
+        if not any([data, metadata, name, description, max_execution_time, spec]):
+            warnings.warn("None of the 'data', 'metadata', 'name', 'description', "
+                          "'max_execution_time', or 'spec' parameters is specified. "
+                          "No update is made.")
+            return
+
+        if data:
+            if "def main(" not in data:
+                # This is the program file
+                with open(data, "r") as file:
+                    data = file.read()
+            data = to_base64_string(data)
+
+        if metadata:
+            metadata = self._read_metadata(metadata=metadata)
+        combined_metadata = self._merge_metadata(
+            metadata=metadata, name=name, description=description,
+            max_execution_time=max_execution_time, spec=spec)
+
+        try:
+            self._api_client.program_update(
+                program_id, program_data=data, **combined_metadata)
+        except RequestsApiError as ex:
+            if ex.status_code == 404:
+                raise RuntimeProgramNotFound(f"Program not found: {ex.message}") from None
+            raise QiskitRuntimeError(f"Failed to update program: {ex}") from None
+
+        if program_id in self._programs:
+            program = self._programs[program_id]
+            program._refresh()
+
+    def _merge_metadata(
+            self,
+            metadata: Optional[Dict] = None,
+            **kwargs: Any
+    ) -> Dict:
+        """Merge multiple copies of metadata.
+        Args:
+            metadata: Program metadata.
+            **kwargs: Additional metadata fields to overwrite.
+        Returns:
+            Merged metadata.
+        """
+        merged = {}
+        metadata = metadata or {}
+        metadata_keys = ['name', 'max_execution_time', 'description', 'spec']
+        for key in metadata_keys:
+            if kwargs.get(key, None) is not None:
+                merged[key] = kwargs[key]
+            elif key in metadata.keys():
+                merged[key] = metadata[key]
+        return merged
 
     def delete_program(self, program_id: str) -> None:
         """Delete a runtime program.
@@ -416,6 +518,10 @@ class IBMRuntimeService:
                 raise RuntimeJobNotFound(f"Program not found: {ex.message}") from None
             raise QiskitRuntimeError(f"Failed to set program visibility: {ex}") from None
 
+        if program_id in self._programs:
+            program = self._programs[program_id]
+            program._is_public = public
+
     def job(self, job_id: str) -> RuntimeJob:
         """Retrieve a runtime job.
 
@@ -437,32 +543,47 @@ class IBMRuntimeService:
             raise QiskitRuntimeError(f"Failed to delete job: {ex}") from None
         return self._decode_job(response)
 
-    def jobs(self, limit: int = 10, skip: int = 0, pending: bool = None) -> List[RuntimeJob]:
+    def jobs(
+            self,
+            limit: Optional[int] = 10,
+            skip: int = 0,
+            pending: bool = None,
+            program_id: str = None
+    ) -> List[RuntimeJob]:
         """Retrieve all runtime jobs, subject to optional filtering.
 
         Args:
-            limit: Number of jobs to retrieve.
+            limit: Number of jobs to retrieve. ``None`` means no limit.
             skip: Starting index for the job retrieval.
             pending: Filter by job pending state. If ``True``, 'QUEUED' and 'RUNNING'
                 jobs are included. If ``False``, 'DONE', 'CANCELLED' and 'ERROR' jobs
                 are included.
+            program_id: Filter by Program ID.
 
         Returns:
             A list of runtime jobs.
         """
         job_responses = []  # type: List[Dict[str, Any]]
         current_page_limit = limit or 20
+        offset = skip
 
         while True:
-            job_page = self._api_client.jobs_get(
+            jobs_response = self._api_client.jobs_get(
                 limit=current_page_limit,
-                skip=skip,
-                pending=pending)["jobs"]
-            if not job_page:
+                skip=offset,
+                pending=pending,
+                program_id=program_id)
+            job_page = jobs_response["jobs"]
+            # count is the total number of jobs that would be returned if
+            # there was no limit or skip
+            count = jobs_response["count"]
+
+            job_responses += job_page
+
+            if len(job_responses) == count - skip:
                 # Stop if there are no more jobs returned by the server.
                 break
 
-            job_responses += job_page
             if limit:
                 if len(job_responses) >= limit:
                     # Stop if we have reached the limit.
@@ -471,7 +592,7 @@ class IBMRuntimeService:
             else:
                 current_page_limit = 20
 
-            skip += len(job_page)
+            offset += len(job_page)
 
         return [self._decode_job(job) for job in job_responses]
 
