@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2020.
@@ -33,5 +31,9 @@ class IQXJsonEncoder(json.JSONEncoder):
         if isinstance(o, complex):
             return (o.real, o.imag)
         if isinstance(o, ParameterExpression):
-            return float(o)
+            try:
+                return float(o)
+            except (TypeError, RuntimeError):
+                val = complex(o)
+                return val.real, val.imag
         return json.JSONEncoder.default(self, o)
