@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2018, 2020.
@@ -30,7 +28,7 @@ from qiskit.providers.ibmq.utils.utils import RefreshQueue
 from ..ibmqtestcase import IBMQTestCase
 from ..decorators import requires_qe_access, requires_provider
 from ..contextmanagers import custom_envs, no_envs
-from ..http_server import SimpleServer, ServerErrorOnceHandler
+from ..http_server import SimpleServer, ServerErrorOnceHandler, ClientErrorHandler
 
 
 class TestAccountClient(IBMQTestCase):
@@ -43,7 +41,7 @@ class TestAccountClient(IBMQTestCase):
         # pylint: disable=arguments-differ
         super().setUpClass()
         cls.provider = provider
-        cls.access_token = cls.provider._api_client.account_api.session.access_token
+        cls.access_token = cls.provider._api_client.account_api.session._access_token
 
     def setUp(self):
         """Initial test setup."""
@@ -61,42 +59,17 @@ class TestAccountClient(IBMQTestCase):
         self.qc2.measure(qr[1], cr[1])
         self.seed = 73846087
 
+        self.fake_server = None
+
+    def tearDown(self) -> None:
+        """Test level tear down."""
+        super().tearDown()
+        if self.fake_server:
+            self.fake_server.stop()
+
     def _get_client(self):
         """Helper for instantiating an AccountClient."""
-        return AccountClient(self.access_token,
-                             self.provider.credentials)
-
-    def test_job_submit_object_storage(self):
-        """Test running a job against a simulator using object storage."""
-        # Create a Qobj.
-        backend_name = 'ibmq_qasm_simulator'
-        backend = self.provider.get_backend(backend_name)
-        circuit = transpile(self.qc1, backend, seed_transpiler=self.seed)
-        qobj = assemble(circuit, backend, shots=1)
-
-        # Run the job through the AccountClient directly using object storage.
-        client = backend._api_client
-
-        job = client.job_submit(backend_name, qobj.to_dict())
-        job_id = job['job_id']
-        self.assertNotIn('shots', job)
-        self.assertEqual(job['kind'], 'q-object-external-storage')
-
-        # Wait for completion.
-        client.job_final_status(job_id)
-
-        # Fetch results and qobj via object storage.
-        result = client._job_result_object_storage(job_id)
-        qobj_downloaded = client._job_download_qobj_object_storage(job_id)
-
-        self.assertEqual(qobj_downloaded, qobj.to_dict())
-        self.assertEqual(result['status'], 'COMPLETED')
-
-    def test_list_jobs_statuses(self):
-        """Check get status jobs by user authenticated."""
-        client = self._get_client()
-        jobs = client.list_jobs_statuses(limit=2)
-        self.assertEqual(len(jobs), 2)
+        return AccountClient(self.provider.credentials)  # pylint: disable=no-value-for-parameter
 
     def test_exception_message(self):
         """Check exception has proper message."""
@@ -153,18 +126,38 @@ class TestAccountClient(IBMQTestCase):
 
     def test_job_submit_retry(self):
         """Test job submit requests get retried."""
-        backend_name = 'ibmq_qasm_simulator'
-        backend = self.provider.get_backend(backend_name)
-        client = backend._api_client
+        client = self._get_client()
 
         # Send request to local server.
         valid_data = {'id': 'fake_id',
                       'objectStorageInfo': {'uploadUrl': SimpleServer.URL},
                       'job': {'id': 'fake_id'}}
-        SimpleServer(handler_class=ServerErrorOnceHandler, valid_data=valid_data).start()
+        self.fake_server = SimpleServer(handler_class=ServerErrorOnceHandler)
+        self.fake_server.set_good_response(valid_data)
+        self.fake_server.start()
         client.account_api.session.base_url = SimpleServer.URL
 
-        client.job_submit(backend_name, {})
+        client.job_submit('ibmq_qasm_simulator', {})
+
+    def test_client_error(self):
+        """Test client error."""
+        client = self._get_client()
+        self.fake_server = SimpleServer(handler_class=ClientErrorHandler)
+        self.fake_server.start()
+        client.account_api.session.base_url = SimpleServer.URL
+
+        sub_tests = [{'error': 'Bad client input'},
+                     {},
+                     {'bad request': 'Bad client input'},
+                     'Bad client input']
+
+        for err_resp in sub_tests:
+            with self.subTest(response=err_resp):
+                self.fake_server.set_error_response(err_resp)
+                with self.assertRaises(RequestsApiError) as err_cm:
+                    client.backend_status('ibmq_qasm_simulator')
+                if err_resp:
+                    self.assertIn('Bad client input', str(err_cm.exception))
 
 
 class TestAccountClientJobs(IBMQTestCase):
@@ -180,7 +173,7 @@ class TestAccountClientJobs(IBMQTestCase):
         # pylint: disable=arguments-differ
         super().setUpClass()
         cls.provider = provider
-        cls.access_token = cls.provider._api_client.account_api.session.access_token
+        cls.access_token = cls.provider._api_client.account_api.session._access_token
 
         backend_name = 'ibmq_qasm_simulator'
         backend = cls.provider.get_backend(backend_name)
@@ -210,16 +203,6 @@ class TestAccountClientJobs(IBMQTestCase):
         response = self.client.job_get(self.job_id)
         self.assertIn('status', response)
 
-    def test_job_status(self):
-        """Test getting job status."""
-        response = self.client.job_status(self.job_id)
-        self.assertIn('status', response)
-
-    def test_job_final_status_websocket(self):
-        """Test getting a job's final status via websocket."""
-        response = self.client._job_final_status_websocket(self.job_id)
-        self.assertEqual(response.pop('status', None), ApiJobStatus.COMPLETED.value)
-
     def test_job_final_status_polling(self):
         """Test getting a job's final status via polling."""
         status_queue = RefreshQueue(maxsize=1)
@@ -227,20 +210,6 @@ class TestAccountClientJobs(IBMQTestCase):
         self.assertEqual(response.pop('status', None), ApiJobStatus.COMPLETED.value)
         self.assertNotEqual(status_queue.qsize(), 0)
 
-    def test_job_properties(self):
-        """Test getting job properties."""
-        # Force the job to finish.
-        _ = self.client._job_final_status_websocket(self.job_id)
-
-        response = self.client.job_properties(self.job_id)
-        # Since the job is against a simulator, it will have no properties.
-        self.assertFalse(response)
-
-    def test_list_jobs_statuses_limit(self):
-        """Test listing job statuses with a limit."""
-        jobs_raw = self.client.list_jobs_statuses(limit=1)
-        self.assertEqual(len(jobs_raw), 1)
-
     def test_list_jobs_statuses_skip(self):
         """Test listing job statuses with an offset."""
         jobs_raw = self.client.list_jobs_statuses(limit=1, skip=1, extra_filter={
@@ -250,12 +219,6 @@ class TestAccountClientJobs(IBMQTestCase):
         for job in jobs_raw:
             self.assertNotEqual(job['job_id'], self.job_id)
 
-    def test_list_jobs_statuses_filter(self):
-        """Test listing job statuses with a filter."""
-        jobs_raw = self.client.list_jobs_statuses(extra_filter={'id': self.job_id})
-        self.assertEqual(len(jobs_raw), 1)
-        self.assertEqual(jobs_raw[0]['job_id'], self.job_id)
-
 
 class TestAuthClient(IBMQTestCase):
     """Tests for the AuthClient."""
@@ -264,7 +227,7 @@ class TestAuthClient(IBMQTestCase):
     def test_valid_login(self, qe_token, qe_url):
         """Test valid authentication."""
         client = AuthClient(qe_token, qe_url)
-        self.assertTrue(client.base_api.session.access_token)
+        self.assertTrue(client.base_api.session._access_token)
 
     @requires_qe_access
     def test_url_404(self, qe_token, qe_url):
