@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2017, 2018.
@@ -14,9 +12,10 @@
 
 """Backends Filtering Test."""
 
+from unittest import mock
 from datetime import datetime
-from dateutil import tz
 
+from dateutil import tz
 from qiskit.providers.ibmq import least_busy
 from qiskit.providers.ibmq import IBMQError
 
@@ -84,7 +83,8 @@ class TestBackendFilters(IBMQTestCase):
     def test_filter_least_busy_reservation(self):
         """Test filtering by least busy function, with reservations."""
         backend = reservations = None
-        for backend in self.provider.backends(simulator=False, operational=True):
+        for backend in self.provider.backends(simulator=False, operational=True,
+                                              status_msg='active'):
             reservations = backend.reservations()
             if reservations:
                 break
@@ -102,8 +102,50 @@ class TestBackendFilters(IBMQTestCase):
         self.assertEqual(least_busy([backend], None), backend)
 
         backs = [backend]
-        for back in self.provider.backends(simulator=False, operational=True):
+        for back in self.provider.backends(simulator=False, operational=True,
+                                           status_msg='active'):
             if back.name() != backend.name():
                 backs.append(back)
                 break
         self.assertTrue(least_busy(backs, window))
+
+    def test_filter_least_busy_paused(self):
+        """Test filtering by least busy function, with paused backend."""
+        backends = self.provider.backends()
+        if len(backends) < 2:
+            self.skipTest("Test needs at least 2 backends.")
+        paused_backend = backends[0]
+        paused_status = paused_backend.status()
+        paused_status.status_msg = 'internal'
+        paused_status.pending_jobs = 0
+        paused_backend.status = mock.MagicMock(return_value=paused_status)
+
+        least_busy_backend = least_busy(backends)
+        self.assertTrue(least_busy_backend)
+        self.assertNotEqual(least_busy_backend.name(), paused_backend.name())
+        self.assertEqual(least_busy_backend.status().status_msg, 'active')
+
+    def test_filter_min_num_qubits(self):
+        """Test filtering by minimum number of qubits."""
+        filtered_backends = self.provider.backends(
+            min_num_qubits=5, simulator=False,
+            filters=lambda b: b.configuration().quantum_volume >= 10)
+
+        self.assertTrue(filtered_backends)
+        for backend in filtered_backends[:5]:
+            with self.subTest(backend=backend):
+                self.assertGreaterEqual(backend.configuration().n_qubits, 5)
+                self.assertTrue(backend.configuration().quantum_volume, 10)
+
+    def test_filter_input_allowed(self):
+        """Test filtering by input allowed"""
+        subtests = ('job', ['job'], ['job', 'runtime'])
+
+        for input_type in subtests:
+            with self.subTest(input_type=input_type):
+                filtered = self.provider.backends(input_allowed=input_type)
+                self.assertTrue(filtered)
+                if not isinstance(input_type, list):
+                    input_type = [input_type]
+                for backend in filtered[:5]:
+                    self.assertTrue(set(input_type) <= set(backend.configuration().input_allowed))
