@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2019.
@@ -15,13 +13,20 @@
 """IBMQBackend Test."""
 
 from inspect import getfullargspec
-from datetime import timedelta
+from datetime import timedelta, datetime
+import warnings
+from unittest import SkipTest
+from unittest.mock import patch
 
+from qiskit import QuantumCircuit, transpile, assemble
+from qiskit.providers.models import QasmBackendConfiguration
+from qiskit.test.reference_circuits import ReferenceCircuits
 from qiskit.providers.ibmq.ibmqbackend import IBMQBackend
 from qiskit.providers.ibmq.ibmqbackendservice import IBMQBackendService
 
 from ..ibmqtestcase import IBMQTestCase
 from ..decorators import requires_device, requires_provider
+from ..utils import get_pulse_schedule, cancel_job
 
 
 class TestIBMQBackend(IBMQTestCase):
@@ -86,8 +91,6 @@ class TestIBMQBackend(IBMQTestCase):
                 defaults = backend.defaults()
                 if backend.configuration().open_pulse:
                     self.assertIsNotNone(defaults)
-                else:
-                    self.assertIsNone(defaults)
 
     def test_backend_reservations(self):
         """Test backend reservations."""
@@ -103,18 +106,18 @@ class TestIBMQBackend(IBMQTestCase):
 
         reserv = reservations[0]
         self.assertGreater(reserv.duration, 0)
+        self.assertTrue(reserv.mode)
         before_start = reserv.start_datetime - timedelta(seconds=30)
-        # after_start = reserv.start_datetime + timedelta(seconds=30)
+        after_start = reserv.start_datetime + timedelta(seconds=30)
         before_end = reserv.end_datetime - timedelta(seconds=30)
         after_end = reserv.end_datetime + timedelta(seconds=30)
 
         # Each tuple contains the start datetime, end datetime, whether a
         # reservation should be found, and the description.
-        # TODO re-enable sub test 3 after API is updated.
         sub_tests = [
             (before_start, after_end, True, 'before start, after end'),
             (before_start, before_end, True, 'before start, before end'),
-            # (after_start, before_end, True, 'after start, before end'),
+            (after_start, before_end, True, 'after start, before end'),
             (before_start, None, True, 'before start, None'),
             (None, after_end, True, 'None, after end'),
             (before_start, before_start, False, 'before start, before start'),
@@ -134,6 +137,78 @@ class TestIBMQBackend(IBMQTestCase):
                     "Reservation {} found={}, used start datetime {}, end datetime {}".format(
                         reserv, found, start_dt, end_dt))
 
+    def test_run_qobj(self):
+        """Test running a Qobj."""
+        qobj = assemble(transpile(ReferenceCircuits.bell(), self.backend), self.backend)
+        with self.assertWarns(DeprecationWarning):
+            job = self.backend.run(qobj)
+        cancel_job(job)
+
+    def test_backend_options(self):
+        """Test backend options."""
+        provider = self.backend.provider()
+        backends = provider.backends(open_pulse=True, operational=True)
+        if not backends:
+            raise SkipTest('Skipping pulse test since no pulse backend found.')
+
+        backend = backends[0]
+        backend.options.shots = 2048
+        backend.set_options(qubit_lo_freq=[4.9e9, 5.0e9],
+                            meas_lo_freq=[6.5e9, 6.6e9],
+                            meas_level=2)
+        job = backend.run(get_pulse_schedule(backend), meas_level=1, foo='foo')
+        qobj = backend.retrieve_job(job.job_id()).qobj()  # Use retrieved Qobj.
+        self.assertEqual(qobj.config.shots, 2048)
+        # Qobj config freq is in GHz.
+        self.assertEqual(qobj.config.qubit_lo_freq, [4.9, 5.0])
+        self.assertEqual(qobj.config.meas_lo_freq, [6.5, 6.6])
+        self.assertEqual(qobj.config.meas_level, 1)
+        self.assertEqual(qobj.config.foo, 'foo')
+        cancel_job(job)
+
+    def test_sim_backend_options(self):
+        """Test simulator backend options."""
+        provider = self.backend.provider()
+        backend = provider.get_backend('ibmq_qasm_simulator')
+        backend.options.shots = 2048
+        backend.set_options(memory=True)
+        job = backend.run(ReferenceCircuits.bell(), shots=1024, foo='foo')
+        qobj = backend.retrieve_job(job.job_id()).qobj()
+        self.assertEqual(qobj.config.shots, 1024)
+        self.assertTrue(qobj.config.memory)
+        self.assertEqual(qobj.config.foo, 'foo')
+
+    def test_deprecate_id_instruction(self):
+        """Test replacement of 'id' Instructions with 'Delay' instructions."""
+
+        circuit_with_id = QuantumCircuit(2)
+        circuit_with_id.id(0)
+        circuit_with_id.id(0)
+        circuit_with_id.id(1)
+
+        config = QasmBackendConfiguration(
+            basis_gates=['id'],
+            supported_instructions=['delay'],
+            dt=0.25,
+            backend_name='test',
+            backend_version=0.0,
+            n_qubits=1,
+            gates=[],
+            local=False,
+            simulator=False,
+            conditional=False,
+            open_pulse=False,
+            memory=False,
+            max_shots=1,
+            coupling_map=None,
+        )
+
+        with patch.object(self.backend, 'configuration', return_value=config):
+            with self.assertWarnsRegex(DeprecationWarning, r"'id' instruction"):
+                self.backend._deprecate_id_instruction(circuit_with_id)
+
+            self.assertEqual(circuit_with_id.count_ops(), {'delay': 3})
+
 
 class TestIBMQBackendService(IBMQTestCase):
     """Test ibmqbackendservice module."""
@@ -143,14 +218,32 @@ class TestIBMQBackendService(IBMQTestCase):
     def setUpClass(cls, provider):
         """Initial class level setup."""
         # pylint: disable=arguments-differ
+        super().setUpClass()
         cls.provider = provider
+        cls.last_week = datetime.now() - timedelta(days=7)
 
     def test_my_reservations(self):
         """Test my_reservations method"""
-        reservations = self.provider.backends.my_reservations()
+        reservations = self.provider.backend.my_reservations()
         for reserv in reservations:
-            print(reserv)
             for attr in reserv.__dict__:
                 self.assertIsNotNone(
                     getattr(reserv, attr),
                     "Reservation {} is missing attribute {}".format(reserv, attr))
+
+    def test_deprecated_service(self):
+        """Test deprecated backend service module."""
+        ref_job = self.provider.backend.jobs(limit=10, end_datetime=self.last_week)[-1]
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", category=DeprecationWarning)
+            self.provider.backends()
+            self.assertEqual(len(w), 0, "DeprecationWarning issued for provider.backends()")
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", category=DeprecationWarning)
+            _ = self.provider.backends.ibmq_qasm_simulator
+            self.provider.backends.retrieve_job(ref_job.job_id())
+            self.provider.backends.jobs(limit=1, end_datetime=self.last_week)
+            self.provider.backends.my_reservations()
+            self.assertGreaterEqual(len(w), 1)
