@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
 # (C) Copyright IBM 2019, 2020.
@@ -15,7 +13,7 @@
 """Test IBMQJob attributes."""
 
 import time
-from unittest import mock
+from unittest import mock, skip
 from datetime import datetime, timedelta
 import re
 import uuid
@@ -28,17 +26,17 @@ from qiskit.providers.jobstatus import JobStatus, JOB_FINAL_STATES
 from qiskit.providers.ibmq.job.exceptions import IBMQJobFailureError
 from qiskit.providers.ibmq.api.clients.account import AccountClient
 from qiskit.providers.ibmq.exceptions import IBMQBackendValueError, IBMQBackendApiProtocolError
-from qiskit.compiler import assemble, transpile
+from qiskit.compiler import transpile
 
-from ..jobtestcase import JobTestCase
+from ..ibmqtestcase import IBMQTestCase
 from ..decorators import requires_provider, requires_device
 from ..utils import (most_busy_backend, cancel_job, get_large_circuit,
-                     update_job_tags_and_verify, bell_in_qobj, submit_job_bad_shots,
+                     update_job_tags_and_verify, submit_job_bad_shots,
                      submit_job_one_bad_instr)
 from ..fake_account_client import BaseFakeAccountClient, MissingFieldFakeJob
 
 
-class TestIBMQJobAttributes(JobTestCase):
+class TestIBMQJobAttributes(IBMQTestCase):
     """Test IBMQJob instance attributes."""
 
     @classmethod
@@ -49,8 +47,9 @@ class TestIBMQJobAttributes(JobTestCase):
         super().setUpClass()
         cls.provider = provider
         cls.sim_backend = provider.get_backend('ibmq_qasm_simulator')
-        cls.qobj = bell_in_qobj(backend=cls.sim_backend)
-        cls.sim_job = cls.sim_backend.run(cls.qobj, validate_qobj=True)
+        cls.bell = transpile(ReferenceCircuits.bell(), cls.sim_backend)
+        cls.sim_job = cls.sim_backend.run(cls.bell)
+        cls.last_week = datetime.now() - timedelta(days=7)
 
     def setUp(self):
         """Initial test setup."""
@@ -77,44 +76,41 @@ class TestIBMQJobAttributes(JobTestCase):
 
         job_properties = [None]
         large_qx = get_large_circuit(backend=backend)
-        qobj = assemble(transpile(large_qx, backend=backend), backend=backend)
-        job = backend.run(qobj, validate_qobj=True)
+        job = backend.run(transpile(large_qx, backend=backend))
         job.wait_for_final_state(wait=None, callback=_job_callback)
         self.assertIsNotNone(job_properties[0])
 
     def test_job_name(self):
         """Test using job names on a simulator."""
-        qobj = bell_in_qobj(self.sim_backend)
-
         # Use a unique job name
         job_name = str(time.time()).replace('.', '')
-        job = self.sim_backend.run(qobj, job_name=job_name, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_name=job_name)
         job_id = job.job_id()
-        rjob = self.provider.backends.retrieve_job(job_id)
+        rjob = self.provider.backend.retrieve_job(job_id)
         self.assertEqual(rjob.name(), job_name)
 
         # Check using partial matching.
         job_name_partial = job_name[8:]
-        retrieved_jobs = self.provider.backends.jobs(backend_name=self.sim_backend.name(),
-                                                     job_name=job_name_partial)
+        retrieved_jobs = self.provider.backend.jobs(
+            backend_name=self.sim_backend.name(), job_name=job_name_partial,
+            start_datetime=self.last_week)
         self.assertGreaterEqual(len(retrieved_jobs), 1)
         retrieved_job_ids = {job.job_id() for job in retrieved_jobs}
         self.assertIn(job_id, retrieved_job_ids)
 
         # Check using regular expressions.
         job_name_regex = '^{}$'.format(job_name)
-        retrieved_jobs = self.provider.backends.jobs(backend_name=self.sim_backend.name(),
-                                                     job_name=job_name_regex)
+        retrieved_jobs = self.provider.backend.jobs(
+            backend_name=self.sim_backend.name(), job_name=job_name_regex,
+            start_datetime=self.last_week)
         self.assertEqual(len(retrieved_jobs), 1)
         self.assertEqual(job_id, retrieved_jobs[0].job_id())
 
     def test_job_name_update(self):
         """Test changing the name associated with a job."""
-        qobj = bell_in_qobj(backend=self.sim_backend)
-
         # Use a unique job name
         initial_job_name = str(time.time()).replace('.', '')
-        job = self.sim_backend.run(qobj, job_name=initial_job_name, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_name=initial_job_name)
 
         new_names_to_test = [
             '',  # empty string as name.
@@ -133,17 +129,16 @@ class TestIBMQJobAttributes(JobTestCase):
 
     def test_duplicate_job_name(self):
         """Test multiple jobs with the same custom job name using a simulator."""
-        qobj = bell_in_qobj(self.sim_backend)
-
         # Use a unique job name
         job_name = str(time.time()).replace('.', '')
         job_ids = set()
         for _ in range(2):
-            job = self.sim_backend.run(qobj, job_name=job_name, validate_qobj=True)
+            job = self.sim_backend.run(self.bell, job_name=job_name)
             job_ids.add(job.job_id())
 
-        retrieved_jobs = self.provider.backends.jobs(backend_name=self.sim_backend.name(),
-                                                     job_name=job_name)
+        retrieved_jobs = self.provider.backend.jobs(
+            backend_name=self.sim_backend.name(), job_name=job_name,
+            start_datetime=self.last_week)
 
         self.assertEqual(len(retrieved_jobs), 2,
                          "More than 2 jobs retrieved: {}".format(retrieved_jobs))
@@ -169,6 +164,7 @@ class TestIBMQJobAttributes(JobTestCase):
                     self.assertIn('bad_instruction', msg)
                     self.assertIsNotNone(re.search(r'Error code: [0-9]{4}\.$', msg), msg)
 
+    @skip("Skip until aer issue 1214 is fixed")
     def test_error_message_simulator(self):
         """Test retrieving job error messages from a simulator backend."""
         job = submit_job_one_bad_instr(self.sim_backend)
@@ -179,7 +175,7 @@ class TestIBMQJobAttributes(JobTestCase):
         message = job.error_message()
         self.assertIn('Experiment 1: ERROR', message)
 
-        r_message = self.provider.backends.retrieve_job(job.job_id()).error_message()
+        r_message = self.provider.backend.retrieve_job(job.job_id()).error_message()
         self.assertIn('Experiment 1: ERROR', r_message)
 
     def test_error_message_validation(self):
@@ -203,7 +199,8 @@ class TestIBMQJobAttributes(JobTestCase):
         if 'COMPLETED' not in self.sim_job.time_per_step():
             self.sim_job.refresh()
 
-        rjob = self.provider.backends.jobs(db_filter={'id': self.sim_job.job_id()})[0]
+        rjob = self.provider.backend.jobs(
+            db_filter={'id': self.sim_job.job_id()}, start_datetime=self.last_week)[0]
         self.assertFalse(rjob._time_per_step)
         rjob.refresh()
         self.assertEqual(rjob._time_per_step, self.sim_job._time_per_step)
@@ -212,7 +209,7 @@ class TestIBMQJobAttributes(JobTestCase):
         """Test retrieving creation date, while ensuring it is in local time."""
         # datetime, before running the job, in local time.
         start_datetime = datetime.now().replace(tzinfo=tz.tzlocal()) - timedelta(seconds=1)
-        job = self.sim_backend.run(self.qobj, validate_qobj=True)
+        job = self.sim_backend.run(self.bell)
         job.result()
         # datetime, after the job is done running, in local time.
         end_datetime = datetime.now().replace(tzinfo=tz.tzlocal()) + timedelta(seconds=1)
@@ -226,7 +223,7 @@ class TestIBMQJobAttributes(JobTestCase):
         """Test retrieving time per step, while ensuring the date times are in local time."""
         # datetime, before running the job, in local time.
         start_datetime = datetime.now().replace(tzinfo=tz.tzlocal()) - timedelta(seconds=1)
-        job = self.sim_backend.run(self.qobj, validate_qobj=True)
+        job = self.sim_backend.run(self.bell)
         job.result()
         # datetime, after the job is done running, in local time.
         end_datetime = datetime.now().replace(tzinfo=tz.tzlocal()) + timedelta(seconds=1)
@@ -238,7 +235,8 @@ class TestIBMQJobAttributes(JobTestCase):
                             'between the start date time {} and end date time {}'
                             .format(step, time_data, start_datetime, end_datetime))
 
-        rjob = self.provider.backends.jobs(db_filter={'id': job.job_id()})[0]
+        rjob = self.provider.backend.jobs(
+            db_filter={'id': job.job_id()}, start_datetime=self.last_week)[0]
         self.assertTrue(rjob.time_per_step())
 
     def test_new_job_attributes(self):
@@ -248,11 +246,10 @@ class TestIBMQJobAttributes(JobTestCase):
             submit_info.update({'batman': 'bruce'})
             return submit_info
 
-        qobj = bell_in_qobj(self.sim_backend)
         original_submit = self.sim_backend._api_client.job_submit
         with mock.patch.object(AccountClient, 'job_submit',
                                side_effect=_mocked__api_job_submit):
-            job = self.sim_backend.run(qobj, validate_qobj=True)
+            job = self.sim_backend.run(self.bell)
 
         self.assertEqual(job.batman_, 'bruce')
 
@@ -260,9 +257,8 @@ class TestIBMQJobAttributes(JobTestCase):
         """Test retrieving queue information."""
         # Find the most busy backend.
         backend = most_busy_backend(self.provider)
-        qobj = bell_in_qobj(backend)
         leave_states = list(JOB_FINAL_STATES) + [JobStatus.RUNNING]
-        job = backend.run(qobj, validate_qobj=True)
+        job = backend.run(self.bell)
         queue_info = None
         for _ in range(20):
             queue_info = job.queue_info()
@@ -292,28 +288,73 @@ class TestIBMQJobAttributes(JobTestCase):
         # Cancel job so it doesn't consume more resources.
         cancel_job(job)
 
-    def test_invalid_job_share_level(self):
-        """Test setting a non existent share level for a job."""
-        with self.assertRaises(IBMQBackendValueError) as context_manager:
-            self.sim_backend.run(self.qobj, job_share_level='invalid_job_share_level',
-                                 validate_qobj=True)
-        self.assertIn('not a valid job share', context_manager.exception.message)
+    def test_esp_readout_not_enabled(self):
+        """Test that an error is thrown is ESP readout is used and the backend does not support it.
+        """
+        saved_api = self.sim_backend._api_client
+        try:
+            self.sim_backend._api_client = BaseFakeAccountClient()
+            # sim backend does not have ``measure_esp_enabled`` flag: defaults to ``False``
+            with self.assertRaises(IBMQBackendValueError) as context_manager:
+                self.sim_backend.run(self.bell, use_measure_esp=True)
+            self.assertIn(
+                "ESP readout not supported on this device. Please make sure the flag "
+                "'use_measure_esp' is unset or set to 'False'.",
+                context_manager.exception.message,
+            )
+        finally:
+            self.sim_backend._api_client = saved_api
 
-    def test_share_job_in_project(self):
-        """Test successfully sharing a job within a shareable project."""
-        job = self.sim_backend.run(self.qobj, job_share_level='project', validate_qobj=True)
+    def test_esp_readout_enabled(self):
+        """Test that ESP readout can be used when the backend supports it."""
+        saved_api = self.sim_backend._api_client
+        try:
+            self.sim_backend._api_client = BaseFakeAccountClient()
+            setattr(self.sim_backend._configuration, "measure_esp_enabled", True)
+            job = self.sim_backend.run(self.bell, use_measure_esp=True)
+            self.assertEqual(job.backend_options()["use_measure_esp"], True)
+        finally:
+            delattr(self.sim_backend._configuration, "measure_esp_enabled")
+            self.sim_backend._api_client = saved_api
 
-        retrieved_job = self.sim_backend.retrieve_job(job.job_id())
-        self.assertEqual(retrieved_job.share_level(), 'project',
-                         "Job {} has incorrect share level".format(job.job_id()))
+    def test_esp_readout_default_value(self):
+        """Test that ESP readout is set to backend support value if not specified."""
+        saved_api = self.sim_backend._api_client
+        try:
+            self.sim_backend._api_client = BaseFakeAccountClient()
+            # ESP readout not enabled on backend
+            setattr(self.sim_backend._configuration, "measure_esp_enabled", False)
+            job = self.sim_backend.run(self.bell)
+            self.assertEqual(job.backend_options()["use_measure_esp"], False)
+            # ESP readout enabled on backend
+            setattr(self.sim_backend._configuration, "measure_esp_enabled", True)
+            job = self.sim_backend.run(self.bell)
+            self.assertEqual(job.backend_options()["use_measure_esp"], True)
+        finally:
+            delattr(self.sim_backend._configuration, "measure_esp_enabled")
+            self.sim_backend._api_client = saved_api
+
+    def test_esp_readout_enabled_not_used(self):
+        """Test that ESP readout is not used if user sets to ``False``, even if backend supports it.
+        """
+        saved_api = self.sim_backend._api_client
+        try:
+            self.sim_backend._api_client = BaseFakeAccountClient()
+            setattr(self.sim_backend._configuration, "measure_esp_enabled", True)
+            job = self.sim_backend.run(self.bell, use_measure_esp=False)
+            self.assertEqual(job.backend_options()["use_measure_esp"], False)
+        finally:
+            delattr(self.sim_backend._configuration, "measure_esp_enabled")
+            self.sim_backend._api_client = saved_api
 
     def test_job_tags_or(self):
         """Test using job tags with an or operator."""
         # Use a unique tag.
         job_tags = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
-        job = self.sim_backend.run(self.qobj, job_tags=job_tags, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=job_tags)
 
-        rjobs = self.sim_backend.jobs(job_tags=['phantom_tag'])
+        rjobs = self.sim_backend.jobs(
+            job_tags=['phantom_tag'], start_datetime=self.last_week)
         self.assertEqual(len(rjobs), 0,
                          "Expected job {}, got {}".format(job.job_id(), rjobs))
 
@@ -321,7 +362,7 @@ class TestIBMQJobAttributes(JobTestCase):
         tags_to_check = [job_tags, job_tags[1:2], job_tags[0:1]+['phantom_tag']]
         for tags in tags_to_check:
             with self.subTest(tags=tags):
-                rjobs = self.sim_backend.jobs(job_tags=tags)
+                rjobs = self.sim_backend.jobs(job_tags=tags, start_datetime=self.last_week)
                 self.assertEqual(len(rjobs), 1,
                                  "Expected job {}, got {}".format(job.job_id(), rjobs))
                 self.assertEqual(rjobs[0].job_id(), job.job_id())
@@ -331,18 +372,20 @@ class TestIBMQJobAttributes(JobTestCase):
         """Test using job tags with an and operator."""
         # Use a unique tag.
         job_tags = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
-        job = self.sim_backend.run(self.qobj, job_tags=job_tags, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=job_tags)
 
         no_rjobs_tags = [job_tags[0:1]+['phantom_tags'], ['phantom_tag']]
         for tags in no_rjobs_tags:
-            rjobs = self.sim_backend.jobs(job_tags=tags, job_tags_operator="AND")
+            rjobs = self.sim_backend.jobs(
+                job_tags=tags, job_tags_operator="AND", start_datetime=self.last_week)
             self.assertEqual(len(rjobs), 0,
                              "Expected job {}, got {}".format(job.job_id(), rjobs))
 
         has_rjobs_tags = [job_tags, job_tags[1:3]]
         for tags in has_rjobs_tags:
             with self.subTest(tags=tags):
-                rjobs = self.sim_backend.jobs(job_tags=tags, job_tags_operator="AND")
+                rjobs = self.sim_backend.jobs(
+                    job_tags=tags, job_tags_operator="AND", start_datetime=self.last_week)
                 self.assertEqual(len(rjobs), 1,
                                  "Expected job {}, got {}".format(job.job_id(), rjobs))
                 self.assertEqual(rjobs[0].job_id(), job.job_id())
@@ -351,7 +394,7 @@ class TestIBMQJobAttributes(JobTestCase):
     def test_job_tags_replace(self):
         """Test updating job tags by replacing a job's existing tags."""
         initial_job_tags = [uuid.uuid4().hex]
-        job = self.sim_backend.run(self.qobj, job_tags=initial_job_tags, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=initial_job_tags)
 
         tags_to_replace_subtests = [
             [],  # empty tags.
@@ -366,7 +409,7 @@ class TestIBMQJobAttributes(JobTestCase):
     def test_job_tags_add(self):
         """Test updating job tags by adding to a job's existing tags."""
         initial_job_tags = [uuid.uuid4().hex]
-        job = self.sim_backend.run(self.qobj, job_tags=initial_job_tags, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=initial_job_tags)
 
         tags_to_add_subtests = [
             [],  # empty tags.
@@ -381,7 +424,7 @@ class TestIBMQJobAttributes(JobTestCase):
     def test_job_tags_remove(self):
         """Test updating job tags by removing from a job's existing tags."""
         initial_job_tags = [uuid.uuid4().hex, uuid.uuid4().hex, uuid.uuid4().hex]
-        job = self.sim_backend.run(self.qobj, job_tags=initial_job_tags, validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=initial_job_tags)
 
         tags_to_remove_subtests = [
             [],
@@ -404,7 +447,7 @@ class TestIBMQJobAttributes(JobTestCase):
 
     def test_job_tags_replace_and_remove(self):
         """Test updating job tags by replacing and removing tags."""
-        job = self.sim_backend.run(self.qobj, job_tags=[uuid.uuid4().hex], validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=[uuid.uuid4().hex])
 
         replacement_tags = [uuid.uuid4().hex, uuid.uuid4().hex]
         # Remove the first tag in `replacement_tags`
@@ -419,7 +462,7 @@ class TestIBMQJobAttributes(JobTestCase):
 
     def test_job_tags_all_parameters(self):
         """Test updating job tags by replacing, adding, and removing tags."""
-        job = self.sim_backend.run(self.qobj, job_tags=[uuid.uuid4().hex], validate_qobj=True)
+        job = self.sim_backend.run(self.bell, job_tags=[uuid.uuid4().hex])
 
         replacement_tags = [uuid.uuid4().hex, uuid.uuid4().hex]
         additional_tags = [uuid.uuid4().hex, uuid.uuid4().hex]
@@ -438,7 +481,7 @@ class TestIBMQJobAttributes(JobTestCase):
     def test_invalid_job_tags(self):
         """Test using job tags with an and operator."""
         self.assertRaises(IBMQBackendValueError, self.sim_backend.run,
-                          self.qobj, job_tags={'foo'})
+                          self.bell, job_tags={'foo'})
         self.assertRaises(IBMQBackendValueError, self.sim_backend.jobs, job_tags=[1, 2, 3])
 
     def test_run_mode(self):
@@ -458,7 +501,7 @@ class TestIBMQJobAttributes(JobTestCase):
         saved_api = self.sim_backend._api_client
         try:
             self.sim_backend._api_client = BaseFakeAccountClient(job_class=MissingFieldFakeJob)
-            self.assertRaises(IBMQBackendApiProtocolError, self.sim_backend.run, self.qobj)
+            self.assertRaises(IBMQBackendApiProtocolError, self.sim_backend.run, self.bell)
         finally:
             self.sim_backend._api_client = saved_api
 
@@ -466,3 +509,16 @@ class TestIBMQJobAttributes(JobTestCase):
         """Test job client version information."""
         self.assertIsNotNone(self.sim_job.result().client_version)
         self.assertIsNotNone(self.sim_job.client_version)
+
+    def test_experiment_id(self):
+        """Test job experiment id."""
+        exp_id = uuid.uuid4().hex
+        job = self.sim_backend.run(self.bell, experiment_id=exp_id)
+        self.assertEqual(job.experiment_id, exp_id)
+        rjob = self.provider.backend.retrieve_job(job.job_id())
+        self.assertEqual(rjob.experiment_id, exp_id)
+        rjobs = self.provider.backend.jobs(experiment_id=exp_id)
+        for rjob in rjobs:
+            self.assertEqual(rjob.experiment_id, exp_id,
+                             f"Job {rjob.job_id()} has experiment ID {rjob.experiment_id} "
+                             f"instead of {exp_id}")
