@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-
 # This code is part of Qiskit.
 #
-# (C) Copyright IBM 2017, 2018.
+# (C) Copyright IBM 2017, 2021.
 #
 # This code is licensed under the Apache License, Version 2.0. You may
 # obtain a copy of this license in the LICENSE.txt file in the root directory
@@ -17,13 +15,15 @@
 from datetime import datetime
 
 from qiskit import ClassicalRegister, QuantumCircuit, QuantumRegister
+from qiskit.test import providers, slow_test
+from qiskit.compiler import transpile
 from qiskit.providers.exceptions import QiskitBackendNotFoundError
+from qiskit.providers.models.backendproperties import BackendProperties
 from qiskit.providers.ibmq.accountprovider import AccountProvider
 from qiskit.providers.ibmq.ibmqbackend import IBMQSimulator, IBMQBackend
-from qiskit.qobj import QobjHeader
-from qiskit.test import providers, slow_test
-from qiskit.compiler import assemble, transpile
-from qiskit.providers.models.backendproperties import BackendProperties
+from qiskit.providers.ibmq.ibmqbackendservice import IBMQBackendService
+from qiskit.providers.ibmq.experiment import IBMExperimentService
+from qiskit.providers.ibmq.random.ibmqrandomservice import IBMQRandomService
 
 from ..decorators import requires_provider, requires_device
 from ..ibmqtestcase import IBMQTestCase
@@ -89,47 +89,41 @@ class TestAccountProvider(IBMQTestCase, providers.ProviderTestCase):
 
     def test_qobj_headers_in_result_sims(self):
         """Test that the qobj headers are passed onto the results for sims."""
-        backends = self.provider.backends(simulator=True)
+        backend = self.provider.get_backend('ibmq_qasm_simulator')
 
         custom_qobj_header = {'x': 1, 'y': [1, 2, 3], 'z': {'a': 4}}
+        circuits = transpile(self.qc1, backend=backend)
 
-        for backend in backends:
-            with self.subTest(backend=backend):
-                circuits = transpile(self.qc1, backend=backend)
+        # TODO Use circuit metadata for individual header when terra PR-5270 is released.
+        # qobj.experiments[0].header.some_field = 'extra info'
 
-                qobj = assemble(circuits, backend=backend)
-                # Update the Qobj header.
-                qobj.header = QobjHeader.from_dict(custom_qobj_header)
-                qobj.experiments[0].header.some_field = 'extra info'
-
-                result = backend.run(qobj, validate_qobj=True).result()
-                self.assertEqual(result.header.to_dict(), custom_qobj_header)
-                self.assertEqual(result.results[0].header.some_field,
-                                 'extra info')
+        job = backend.run(circuits, qobj_header=custom_qobj_header)
+        result = job.result()
+        self.assertTrue(custom_qobj_header.items() <= job.header().items())
+        self.assertTrue(custom_qobj_header.items() <= result.header.to_dict().items())
+        # self.assertEqual(result.results[0].header.some_field,
+        #                  'extra info')
 
     @slow_test
     @requires_device
     def test_qobj_headers_in_result_devices(self, backend):
         """Test that the qobj headers are passed onto the results for devices."""
-        # pylint: disable=unused-argument
         custom_qobj_header = {'x': 1, 'y': [1, 2, 3], 'z': {'a': 4}}
 
-        qobj = assemble(transpile(self.qc1, backend=backend), backend=backend)
-        # Update the Qobj header.
-        qobj.header = QobjHeader.from_dict(custom_qobj_header)
-        # Update the Qobj.experiment header.
-        qobj.experiments[0].header.some_field = 'extra info'
+        # TODO Use circuit metadata for individual header when terra PR-5270 is released.
+        # qobj.experiments[0].header.some_field = 'extra info'
 
-        job = backend.run(qobj, validate_qobj=True)
+        job = backend.run(transpile(self.qc1, backend=backend), qobj_header=custom_qobj_header)
         job.wait_for_final_state(wait=300, callback=self.simple_job_callback)
         result = job.result()
-        self.assertEqual(result.header.to_dict(), custom_qobj_header)
-        self.assertEqual(result.results[0].header.some_field,
-                         'extra info')
+        self.assertTrue(custom_qobj_header.items() <= job.header().items())
+        self.assertTrue(custom_qobj_header.items() <= result.header.to_dict().items())
+        # self.assertEqual(result.results[0].header.some_field,
+        #                  'extra info')
 
     def test_aliases(self):
         """Test that display names of devices map the regular names."""
-        aliased_names = self.provider.backends._aliased_backend_names()
+        aliased_names = self.provider.backend._aliased_backend_names()
 
         for display_name, backend_name in aliased_names.items():
             with self.subTest(display_name=display_name,
@@ -166,3 +160,18 @@ class TestAccountProvider(IBMQTestCase, providers.ProviderTestCase):
                              if isinstance(getattr(self.provider.backends, back), IBMQBackend)}
         backends = {back.name().lower() for back in self.provider._backends.values()}
         self.assertEqual(provider_backends, backends)
+
+    def test_provider_services(self):
+        """Test provider services."""
+        services = self.provider.services()
+        self.assertIn('backend', services)
+        self.assertIsInstance(services['backend'], IBMQBackendService)
+        self.assertIsInstance(self.provider.service('backend'), IBMQBackendService)
+        self.assertIsInstance(self.provider.backend, IBMQBackendService)
+
+        if 'experiment' in services:
+            self.assertIsInstance(self.provider.service('experiment'), IBMExperimentService)
+            self.assertIsInstance(self.provider.experiment, IBMExperimentService)
+        if 'random' in services:
+            self.assertIsInstance(self.provider.service('random'), IBMQRandomService)
+            self.assertIsInstance(self.provider.random, IBMQRandomService)
